@@ -31,6 +31,8 @@ construct.scales <-
              labels = FALSE,
              col = FALSE,
              col.line = col,
+             alpha = FALSE,
+             alpha.line = alpha,
              log = FALSE,
              font = FALSE,
              fontfamily = FALSE,
@@ -54,6 +56,7 @@ construct.scales <-
                  fontfamily = fontfamily, fontface = fontface, 
                  at = at, labels = labels,
                  col = col, col.line = col.line,
+                 alpha = alpha, alpha.line = alpha.line,
                  log = log,
                  alternating = alternating,
                  relation = relation,
@@ -105,6 +108,8 @@ construct.3d.scales <-
              labels = FALSE,
              col = FALSE,
              col.line = col,
+             alpha = FALSE,
+             alpha.line = col,
              log = FALSE,
              font = FALSE,
              fontfamily = FALSE,
@@ -126,6 +131,7 @@ construct.3d.scales <-
                  fontfamily = fontfamily, fontface = fontface, 
                  at = at, labels = labels,
                  col = col, col.line = col.line,
+                 alpha = alpha, alpha.line = alpha.line,
                  log = log, arrows = arrows,
                  relation = relation, format = format,
                  abbreviate = abbreviate, minlength = minlength)
@@ -159,13 +165,13 @@ construct.3d.scales <-
 
 
 
-
-
-limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
+limitsFromLimitlist <- function(have.lim, lim, relation, limitlist,
+                                used.at, numlimitlist, axs, nplots)
     ## have.lim: logical, whether xlim/ylim was explicitly specified
     ## lim: the specified limit if have.lim = TRUE
     ## relation: same/free/sliced
     ## limitlist: list of limits from prepanel calculations, one for each panel
+    ## numlimitlist: (optional) numeric locations for factors (lim will be levels including unused ones)
     ## axs: "r", "i" etc, passed on to extend.limits
 
     ## return value depends on relation. (See limits.and.aspect below
@@ -195,7 +201,8 @@ limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
         ## class.
 
 
-        if (have.lim) {
+        if (have.lim)
+        {
             if (is.list(lim)) stop("limits cannot be a list when relation = same")
             limits <- lim
             slicelen <- if (is.numeric(lim)) diff(range(lim)) else length(lim) + 2
@@ -212,23 +219,24 @@ limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
 
             limits <- unlist(limitlist) ## loses the class attribute
 
-            if (length(limits) > 0) {
-
-                if (is.numeric(limits)) {
-
+            if (length(limits) > 0)
+            {
+                if (is.numeric(limits))
+                {
                     limits <- extend.limits(range(limits, na.rm = TRUE), axs = axs)
                     slicelen <- diff(range(limits))
-
                 }
-                else {
+                else
+                {
                     limits <- unique(limits[!is.na(limits)])
                     slicelen <- length(limits) + 2
                 }
 
                 ## hopefully put back appropriate class of limits:
 
-                if (length(class.lim) > 0) class(limits) <-
-                    if (all(class.lim[[1]] == "integer")) "numeric" else class.lim[[1]]
+                if (length(class.lim) > 0)
+                    class(limits) <-
+                        if (all(class.lim[[1]] == "integer")) "numeric" else class.lim[[1]]
 
                 ## (have to handle "integer" specially, since variable
                 ## specifications like 1:10 are rather common, and
@@ -236,7 +244,8 @@ limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
                 ## integers)
 
             }
-            else {
+            else
+            {
                 limits <- c(0,1)
                 slicelen <- 1
             }
@@ -247,10 +256,12 @@ limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
     }
 
 
-    else if (relation == "sliced") {
-
-        if (have.lim) {
-            if (is.list(lim)) {
+    else if (relation == "sliced")
+    {
+        if (have.lim)
+        {
+            if (is.list(lim))
+            {
                 limits <- rep(lim, length = nplots)
             }
             else warning("Explicitly specified limits ignored")
@@ -260,21 +271,31 @@ limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
             slicelen[[i]] <-
                 if (is.numeric(limitlist[[i]]))
                     diff(range(limitlist[[i]])) # range unnecessary, but...
+                else if (!any(is.na(numlimitlist[[i]])))
+                    diff(range(numlimitlist[[i]]))
                 else NA
         slicelen <- (if (axs == "i") 1 else 1.14) * max(unlist(slicelen), na.rm = TRUE)
-        for (i in seq(along = limitlist)) {
+        for (i in seq(along = limitlist))
+        {
             if (is.numeric(limitlist[[i]]))
                 limitlist[[i]] <-
                     extend.limits(limitlist[[i]], length = slicelen)
         }
-        ans <- list(limits = limitlist, slicelen = slicelen)
+        for (i in seq(along = numlimitlist))
+        {
+            if (!all(is.na(numlimitlist[[i]])))
+                numlimitlist[[i]] <-
+                    extend.limits(numlimitlist[[i]], length = slicelen)
+        }
+        ans <- list(limits = limitlist, used.at = used.at,
+                    numlimitlist = numlimitlist, slicelen = slicelen)
     }
 
 
     else if (relation == "free") {
 
-        if (have.lim) {
-
+        if (have.lim)
+        {
             ## FIXME: What's all this about ? I seem to have forgotten
             ## :-( what should happen if xlim is specified ? Should
             ## that override "free" ? Make sure docs are consistent
@@ -290,12 +311,14 @@ limitsFromLimitlist <- function(have.lim, lim, relation, limitlist, axs, nplots)
             limitlist[id] <- lim
         }
 
-        for (i in seq(along = limitlist)) {
+        for (i in seq(along = limitlist))
+        {
             if (is.numeric(limitlist[[i]])) 
                 limitlist[[i]] <- extend.limits(limitlist[[i]], axs = axs) ## preserves class
             ## o.w., keep it as it is
         }
-        ans <- list(limits = limitlist, slicelen = 1)
+        ans <- list(limits = limitlist, used.at = used.at,
+                    numlimitlist = numlimitlist, slicelen = 1)
     }
 
     ans
@@ -335,6 +358,10 @@ limits.and.aspect <-
     if (nplots<1) stop("need at least one panel")
     x.limits <- vector("list", nplots)
     y.limits <- vector("list", nplots)
+    x.used.at <- vector("list", nplots)
+    y.used.at <- vector("list", nplots)
+    x.num.limit <- vector("list", nplots)
+    y.num.limit <- vector("list", nplots)
     dxdy <- vector("list", nplots)
 
     for (count in seq(length = nplots))
@@ -348,16 +375,25 @@ limits.and.aspect <-
                 prenames <- names(formals(prepanel))
                 if (!("..." %in% prenames)) pargs <- pargs[prenames]
                 pretem <- do.call("prepanel", pargs)
-                tem[names(pretem)] <- pretem
+                tem <- updateList(tem, pretem)
+                ## tem[names(pretem)] <- pretem
             }
             x.limits[[count]] <- tem$xlim
             y.limits[[count]] <- tem$ylim
+            x.used.at[[count]] <- if (is.null(tem$xat)) NA else tem$xat
+            y.used.at[[count]] <- if (is.null(tem$yat)) NA else tem$yat
+            x.num.limit[[count]] <- if (is.null(tem$xat)) NA else range(tem$xat)
+            y.num.limit[[count]] <- if (is.null(tem$yat)) NA else range(tem$yat)
             dxdy[[count]] <- list(tem$dx, tem$dy)
         }
         else  ## this happens for empty panels
         {
             x.limits[[count]] <- c(NA, NA)
             y.limits[[count]] <- c(NA, NA)
+            x.used.at[[count]] <- NA
+            y.used.at[[count]] <- NA
+            x.num.limit[[count]] <- NA
+            y.num.limit[[count]] <- NA
             dxdy[[count]] <- list(NA, NA)
         }
     }
@@ -377,6 +413,8 @@ limits.and.aspect <-
                             lim = xlim,
                             relation = x.relation,
                             limitlist = x.limits,
+                            used.at = x.used.at,
+                            numlimitlist = x.num.limit,
                             axs = x.axs,
                             nplots = nplots)
 
@@ -386,25 +424,38 @@ limits.and.aspect <-
                             lim = ylim,
                             relation = y.relation,
                             limitlist = y.limits,
+                            used.at = y.used.at,
+                            numlimitlist = y.num.limit,
                             axs = y.axs,
                             nplots = nplots)
 
     if (is.character(aspect))
-
-        if (aspect == "xy") {
-
+    {
+        if (aspect == "xy")
+        {
+#print(dxdy)
             aspect <- median(unlist(lapply(dxdy, banking)),
                              na.rm = TRUE) * y.limits$slicelen /
                                  x.limits$slicelen
-
-            if (y.relation == "free")
-                warning("aspect=xy when y-relation=free is not sensible")
-
+#print(aspect)
+            if (y.relation == "free" || x.relation == "free")
+                warning("aspect=xy when relation=free is not sensible")
+        }
+        else if (aspect == "iso")
+        {
+            aspect <- y.limits$slicelen / x.limits$slicelen
+            if (y.relation == "free" || x.relation == "free")
+                warning("aspect=iso when relation=free is not sensible")
         }
         else aspect <- 1
+    }
 
     list(x.limits = x.limits$limits,
          y.limits = y.limits$limits,
+         x.used.at = x.limits$used.at,
+         y.used.at = y.limits$used.at,
+         x.num.limit = x.limits$numlimitlist,
+         y.num.limit = y.limits$numlimitlist,
          aspect.ratio = aspect,
          prepanel = prepanel)
 }
